package CustomFieldDataChecker::CMS;

use strict;
use CustomFields::Util qw( get_meta );

sub CMSPreSave_customfield_objs {
    my ($cb, $app, $obj) = @_;
    return 1 unless $app->isa('MT::App');
    my $q = $app->param;
    my $blog_id = $app->param('blog_id') || 0;

    return 1 if !$q->param('customfield_beacon');

    my $plugin = MT->component("CustomFieldDataChecker");
    my $basename_list = $plugin->get_config_value('check_data', 'blog:'.$app->param('blog_id'));
    return 1 if !$basename_list;

    my @basename;
    if ($basename_list =~ /,/) {
        $basename_list =~ s/\s*//g;
        @basename = split(/,/, $basename_list);
    } else {
        push(@basename, $basename_list);
    }

    require CustomFields::Field;
    my $meta = {};
    foreach ($q->param()) {
        next if $_ eq 'customfield_beacon';
        if (m/^customfield_(.*?)$/) {
            my $field_name = $1;

            my $name;
            my $flag = 0;
            my $basename;
            foreach my $tmp (@basename) {
                if ($tmp eq $field_name) {
                    $basename = $tmp;
                    $flag = 1;
                    last;
                }
            }
            next unless $flag;

            if ( m/^customfield_(.+?)_cb_beacon$/ ) {
                $field_name = $1;
                my $field = CustomFields::Field->load(
                  {
                    $blog_id ? ( blog_id => [ $blog_id, 0 ] ) : ( blog_id => $blog_id ),
                    basename => $field_name
                  }
                ) or next;
                $name = $field->name;

                $meta->{$field_name} = defined( $q->param("customfield_$field_name") )
                  ? $q->param("customfield_$field_name")
                  : '0';
            }
            else {
                my $field = CustomFields::Field->load(
                  {
                    $blog_id ? ( blog_id => [ $blog_id, 0 ] ) : ( blog_id => $blog_id ),
                    basename => $field_name
                  }
                ) or next;
                $name = $field->name;

                if ($field->type eq 'datetime') {
                    $meta->{$field_name} =
                        $q->param("customfield_$field_name") ne ''
                        ? $q->param("customfield_$field_name")
                        : undef;
                } else {
                    $meta->{$field_name} =
                        $q->param("customfield_$field_name");
                }
            }
            my $class_name = $obj->class;
            my $iter = MT->model($class_name)->load_iter({ blog_id => $app->param('blog_id') });
            while ( my $entry = $iter->() ) {
                next if $entry->id == $obj->id;
                my $meta_data = get_meta($entry);
                next if !$meta_data->{$field_name} && !$meta->{$field_name};
                return $app->error( $app->translate('duplicate data [_1][_2][_3]', $name, $entry->title, $entry->class_label) )
                    if $meta_data->{$field_name} eq $meta->{$field_name};
            }
        }
    }
    1;
}

1;
